
##### ASSIGNMENT 3 - WRITING AN R FUNCTION #####


# GETTING STARTED

rm(list=ls())   #delete all variables
library(shiny)  #load package

#Set Your working directory to the A3_Sarah_Mueller- folder.

## IMPORTANT ##
#do NOT move the files in the folder!

######################################################################################################################################

# WHAT I WANT TO DO

# I want to write a function with which the user can easily calculate either the area or the volume of different objects. To make it easier
# to type the input in, the code is embeded in an App built with the shiny package.
# First, the user needs to define which object (triangle, circle, etc.) needs to be considered and what should be calculated (area, volume)
# After that, the user can type in the known values (e.g. side lengths etc.).
# The user then gets the required output.
# Additionally, pictures of the objects are plotted and labelled with the names of the input values the user needs to type in.

###

# IMPORTANT!
#to start the App please select Run App in R Studio (optional: select where you want to display it)

###

######################################################################################################################################

# PREPARE THE USER INTERFACE

    ui<- fluidPage( 
        titlePanel("EASY SHAPES"),   # create a title --> Name of the App
        sidebarLayout(    
          sidebarPanel(   #what is on the left hand side of the App
# let the user choose the shape of the object        
        selectInput(
          inputId= "shape",   #ID to identify the input later in the server function below
          label= "What is the shape of your object?",  #question/header visible for user
          choices= c("Rectangle/Square","Rectangular Prism", "Pyramid", "Trapezium", "Kite", "Parallelogram", "Rhombus",  #specify all possible objects/shapes
                     "Circle", "Sphere", "Ellipse", "Ellipsoid", "Cylinder", "Cone", "Isoceles Triangle", "Right Triangle"),
          multiple = FALSE),   #only one at a time is choosable

# let the user decide wether to calculate the volume Area or Surface Area
        selectInput(
          inputId= "volarea",
          label= "What do you want to calculate?",
          choices= c("Volume", "Area", "Surface Area"),   #specify the choices
          multiple= FALSE),   #only one at a time is choosable

# Lets the user type in the variables needed for calculating the Area/Volume
        numericInput(inputId= "a",
                     label= "Length of a",
                     value= NULL),   #set initial value to NULL
        numericInput(inputId= "b",
                    label= "Length of b",
                    value= NULL),
        numericInput(inputId= "c",
                    label= "Length of c",
                    value= NULL),        
        numericInput(inputId= "e",
                  label= "Length of d",
                   value= NULL),
        numericInput(inputId= "d",
                   label= "Value of alpha",
                   value= NULL),

actionButton("go", "Go"),

# specify the unit of the users values
        textInput(inputId= "coef", 
        label="Please specify the unit of your values"),

# Place for some comments for the user and printing the results
        textOutput("comment"),
        p(),  #generates a new paragraph
        strong(textOutput("result_are")),  #output of Result for Area
        strong(textOutput("result_sare")),   #output of Result for Surface Area
        strong(textOutput("result_vol"))),   #output of Result for Volume

        mainPanel(   #what is on the right hand side in the App
          img(src = "shapes.jpg", height = 750, width = 530) # picture of the shapes
       ))
)   # FINISHES DEFINING THE USER INTERFACE
  
    
###########################################################################################################################################

# DEFINE THE SERVER FUNCTION

server<- function(input, output) {

############################# 
  
# Calculate the Area and save it to a variable, then print the result to the window.
  
  are<- reactive({  # save the output under are if Area should be calculated
    if(input$volarea == "Area"){ 
       
      if(input$shape == "Rectangle/Square"){    #calculate area of a rectangle and print result in are
          ar<- input$a * input$b
          print(paste("The Area of your", input$shape,"is",ar, input$coef,"^2."))
        
        
            
      } else if(input$shape == "Circle"){    #calculate area of a circle and print result in are
          ar<- (input$a)^2 *pi
          print(paste("The Area of your", input$shape,"is",ar, input$coef,"^2."))

          
          
      } else if(input$shape == "Ellipse"){    #calculate area of a ellipse and print result in are
        ar<- pi* input$a* input$b
        print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        
        
        
      } else if(input$shape == "Trapezium"){   #calculate area of a trapezium and print result in are 
        ar<- (input$a+ input$b)/2* input$c
        print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        
        
        
      } else if(input$shape == "Parallelogram"){    #calculate area of a parallelogram and print result in are
          if((is.integer(input$a))&(is.integer(input$b))){
            ar<- input$a* input$b
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
          } else if((is.integer(input$a))&(is.integer(input$c))&(is.integer(input$d))){ 
            ar<- input$a* input$c* sin((input$d* pi)/180)
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
          } else{ print("Please enter values for a, b or a, c and alpha")
            
          }
        
        
        
      } else if(input$shape == "Rhombus"){    #calculate area of a rhombus and print result in are
          if((is.integer(input$a))&(is.integer(input$b))){
            ar<- input$a* input$b / 2
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else if((is.integer(input$c))&(is.integer(input$d))){ 
          ar<- input$c^2* sin((input$d* pi)/180)
          print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else{
          print("Please enter values for a, b or c and alpha")
        }
        
        
        
      } else if(input$shape == "Kite"){     #calculate area of a kite and print result in are
          if((is.integer(input$a))&(is.integer(input$b))){
            ar<- input$a* input$b / 2
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else if((is.integer(input$c))&(is.integer(input$e))&(is.integer(input$d))){ 
            ar<- input$c* input$e* sin((input$d* pi)/180)
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else{
            print("Please enter values for a, b or c, d and alpha")
          }
        
        
        
      } else if(input$shape == "Isoceles Triangle"){    #calculate area of a isoceles triangle and print result in are
          if((is.integer(input$a))&(is.integer(input$b))){
            ar<- sqrt(input$b^2- (input$a/2)^2)* input$a/2
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
          } else if((is.integer(input$a))&(is.integer(input$c))){
            ar<- input$c* input$a/2
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
          } else{
            ar<- sqrt(input$b^2 - input$c^2)* input$c
            print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
          }
        
        
      } else if(input$shape == "Right Triangle"){    #calculate area of a right triangle and print result in are
        if((is.integer(input$a))&(is.integer(input$b))){
          ar<- input$a * input$b /2 
          print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else if((is.integer(input$a))&(is.integer(input$c))){
          ar<- sqrt(input$c^2- input$a^2)* input$a/2
          print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        } else{ 
          sqrt(input$c^2- input$b^2)* input$b/2
          print(paste("Area A of your", input$shape, "is", ar, input$coef, "^2."))
        }
        
        
        
          } else{print("Please insert valid object shape")} #if another shape is chosen while Area is chosen print this
  
      
      
    } else{print("")}   #To make sure nothing is printed if Area is not chosen
    
    
})   # End of defining are

  
#############################    
  
 # calculate the Surface Area and save it to a variable, then print it  
 
  sare<- reactive({    #define sare as surface area for the chosen shape
   if(input$volarea == "Surface Area"){
     
     if(input$shape == "Sphere"){    #calculate Surface Area of a Sphere and print result in sare
       sar<- 4* pi * (input$a)^2
       print(paste("The Total Surface Area of your", input$shape, "is", sar, input$coef, "^2"))
     
       
       
    } else if(input$shape == "Rectangular Prism"){    #calculate Surface Area of a rectangular prism and print result in sare
       sar_1<- input$a * input$b
       sar_2<- input$b * input$c
       sar_3<- input$c * input$a
       sar_4<- 2* sar_1+ 2* sar_2+ 2* sar_3
       print(paste("Area A of your", input$shape,"is",sar_1, input$coef,"^2, Area B is", sar_2, input$coef,
                   "^2 and Area C is", sar_3, input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_4, input$coef, "^2."))
       
       
       
    } else if(input$shape == "Cylinder"){    #calculate Surface Area of a cylinder and print result in sare
      sar_1<- input$a^2 * pi
      sar_2<- input$a* pi *2* input$b
      sar_3<- 2* sar_1 + sar_2
      print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                  input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))

      
      
    } else if(input$shape == "Cone"){    #calculate Surface Area of a cone and print result in sare
      if((is.integer(input$a))&(is.integer(input$c))){   #if a and c are given
         sar_1<- pi* input$a^2
         sar_2<- pi* input$a* input$c
         sar_3<- sar_1+ sar_2
         print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                  input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
      } else if((is.integer(input$b))&(is.integer(input$c))){   #if b and c are given
        a<- sqrt(input$c^2 - input$b^2)
        sar_1<- pi* a^2
        sar_2<- pi* a* input$c
        sar_3<- sar_1+ sar_2
        print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                    input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
      } else{   #if a and b are given
        c<- sqrt(input$a^2 + input$b^2)
        sar_1<- pi* input$a^2
        sar_2<- pi* input$a* c
        sar_3<- sar_1+ sar_2
        print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                    input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
      }
      
      
    } else if(input$shape == "Pyramid"){    #calculate Surface Area of a pyramid and print result in sare
        if((is.integer(input$a))&(is.integer(input$b))){
          sar_1<- input$a^2
          sar_2<- sqrt(input$b^2- (input$a/2)^2)* input$a/2
          sar_3<- sar_1 + 4* sar_2
          print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                  input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
        }else if((is.integer(input$b))&(is.integer(input$c))){
          a<- (sqrt(input$b^2 - input$c^2)*2)/sqrt(2)
          sar_1<- a^2
          sar_2<- sqrt(input$b^2- (a/2)^2)* a/2
          sar_3<- sar_1 + 4* sar_2
          print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                      input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
        } else{
          b<- sqrt((input$a*sqrt(2)/2)^2 + input$c^2)
          sar_1<- input$a^2
          sar_2<- sqrt(b^2- (input$a/2)^2)* input$a/2
          sar_3<- sar_1 + 4* sar_2
          print(paste("Area A of your", input$shape, "is", sar_1, input$coef, "^2 and Area B is", sar_2, 
                      input$coef, "^2. The Total Surface Area of your", input$shape, "is", sar_3, input$coef, "^2"))
        }
      
      
       } else{print("Please enter valid object shape")}  #if surface area is chosen but not shape stated above, print this
   
     
     
   } else{print("")}  # to make sure nothing is printed if surface area is not chosen
   
   
   
 })   #end of defining sare
 
 
############################# 
 
# Calculate the Volume and save it to a variable, then print it
 
   vol<- reactive({   #define vol as the volume calculated
  if(input$volarea == "Volume"){
    
    if(input$shape == "Sphere"){   #calculate volume of a Sphere and print result in vol
      vol<- 4/3* pi * (input$a)^3
      print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      
      
      
    } else if(input$shape == "Rectangular Prism"){  #calculate volume of a rectangular prism and print result in vol
      vol<- input$a * input$b * input$c
      print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      
      
      
    } else if(input$shape == "Cylinder"){   #calculate volume of a cylinder and print result in vol
      vol<- pi* input$a^2 * input$b
      print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      
      
      
    } else if(input$shape == "Elipsoid"){  #calculate volume of an ellipsoid and print result in vol
      vol<- 4/3* pi* input$a* input$b* input$c
      print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      
      
      
    } else if(input$shape == "Cone"){   #calculate volume of a cone and print result in vol
      if((is.integer(input$a))&(is.integer(input$b))){
        vol<- 1/3* pi* input$a^2* input$b
        print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      } else if((is.integer(input$b))&(is.integer(input$c))){
        a<- sqrt(input$c^2 - input$b^2)
        vol<- 1/3* pi* a^2* input$b
        print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      } else{
        b<- sqrt(input$c^2 - input$a^2)
        vol<- 1/3* pi* input$a^2* b
        print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      }
      
      
    } else if(input$shape == "Pyramid"){   #calculate volume of a pyramid and print result in vol
      if((is.integer(input$a))&(is.integer(input$c))){
      vol<- 1/3* input$a^2 * input$c
      print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
      } else if((is.integer(input$a))&(is.interger(input$b))){
        c<- sqrt(input$b^2 - ((input$a* sqrt(2)/2)^2))
        vol<- 1/3* input$a^2 * c
        print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))
        
      } else{
        a<- (sqrt(input$b^2 - input$c^2)*2)/sqrt(2)
        vol<- 1/3* a^2 * input$c
        print(paste("The Volume of your", input$shape, "is", vol, input$coef, "^3"))  
      }
      
      
      } else(print ("Please specify valid object shape"))   #if not shape stated aboe is chosen print this text.
    
    
    
   } else(print(""))   #to make sure nothign is printed if volume is not chosen
    
     
     
  })   #end of defining vol

###################################################

# PRINT THE OUTPUT
  
# Display the Result of calculating the Area on the right hand side of the Window
 output$result_are<- renderText({   #prints the text of are where result_are is stated in the ui
   are()  
 })
 
# Display the Result of calculating the Total Surface Area on the right hand side of the window
 output$result_sare<- renderText({   #prints the text of sare where result_sare is stated in the ui
   sare()
 })       
 
#Display the Result of calculating the Volume on the right hand side of the window
 output$result_vol<- renderText({   #prints the text of vol where result_vol is stated in the ui
    vol()
 }) 
 
# Give the user important information regarding usage of units/values
 output$comment<- renderText({
   print("IMPORTANT: All input values must have the same unit (e.g. all m, cm etc.) and
         specify angles in degrees!")
 })


} # END OF SERVER FUNCTION

###################################################################################################################################

# START THE SHINY APP
shinyApp(ui=ui, server=server)  



###################################################################################################################################



